/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package jakarta.el;

/**
 * Wrapper for a {@link Class} object that enables access to static fields and methods from EL expressions. An
 * ELClass instance is used as the base object when resolving static members, typically in conjunction with the
 * {@link StaticFieldELResolver}.
 *
 * @since EL 3.0
 */
public class ELClass {

    private final Class<?> clazz;

    /**
     * Constructs an ELClass wrapping the specified Class.
     *
     * @param clazz the class to wrap
     */
    public ELClass(Class<?> clazz) {
        this.clazz = clazz;
    }

    /**
     * Returns the wrapped Class.
     *
     * @return the wrapped Class
     */
    public Class<?> getKlass() {
        return clazz;
    }
}
