/*
 *  Licensed to the Apache Software Foundation (ASF) under one or more
 *  contributor license agreements.  See the NOTICE file distributed with
 *  this work for additional information regarding copyright ownership.
 *  The ASF licenses this file to You under the Apache License, Version 2.0
 *  (the "License"); you may not use this file except in compliance with
 *  the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.apache.tomcat.util.net.openssl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

/**
 * Represents an OpenSSL configuration containing a list of commands.
 */
public class OpenSSLConf implements Serializable {

    /**
     * Constructs a new OpenSSLConf.
     */
    public OpenSSLConf() {
    }

    private static final long serialVersionUID = 1L;

    /**
     * The list of OpenSSL configuration commands.
     */
    private final List<OpenSSLConfCmd> commands = new ArrayList<>();

    /**
     * Adds a command to this configuration.
     *
     * @param cmd The command to add
     */
    public void addCmd(OpenSSLConfCmd cmd) {
        commands.add(cmd);
    }

    /**
     * Returns the list of commands in this configuration.
     *
     * @return The list of commands
     */
    public List<OpenSSLConfCmd> getCommands() {
        return commands;
    }

}
