/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package jakarta.persistence;

import java.lang.annotation.ElementType;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;

/**
 * Specifies the entity manager for use with an entity bean or other managed class.
 * Used to inject a persistence context into a Jakarta EE managed component.
 */
@Target({ ElementType.TYPE, ElementType.METHOD, ElementType.FIELD })
@Retention(RetentionPolicy.RUNTIME)
public @interface PersistenceContext {
    /**
     * The name used to look up the persistence context.
     *
     * @return the persistence context name
     */
    String name() default "";

    /**
     * The unit name of the persistence unit to associate with the persistence context.
     *
     * @return the persistence unit name
     */
    String unitName() default "";

    /**
     * The type of the persistence context.
     *
     * @return the persistence context type
     */
    PersistenceContextType type() default PersistenceContextType.TRANSACTION;

    /**
     * Properties used to configure the persistence context.
     *
     * @return the configuration properties
     */
    PersistenceProperty[] properties() default {};

    /**
     * The synchronization type of the persistence context with the transaction.
     *
     * @return the synchronization type
     */
    SynchronizationType synchronization() default SynchronizationType.SYNCHRONIZED;
}
